﻿#region MIT Licenze
//The MIT License

//Copyright (c) 2008 Sebastian Faltoni, dotnetfireball.org

//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:

//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.

//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Fireball.OpenXML.Word
{
    /// <summary>
    /// This class store the revision information
    /// for the owner run
    /// </summary>
    public sealed class RunRevisionInformation : OpenXMLEntity
    {
        //author
        public string Author
        {
            get;
            set;
        }

        //date
        public DateTime Date
        {
            get;
            set;
        }

        public decimal Id
        {
            get;
            set;
        }

        // can't be nested
        public StyleRunProperties PreviousProperties
        {
            get;
            set;
        }

        public override System.Xml.Linq.XContainer ToXml()
        {
            XElement xRunInfo = new XElement(NS.WM + "rPrChange");

            xRunInfo.Add(new XAttribute(NS.WM + "author", this.Author));
            xRunInfo.Add(new XAttribute(NS.WM + "date", this.Date.ToUniversalTime().ToString()));
            xRunInfo.Add(new XAttribute(NS.WM + "id", this.Id));

            if (this.PreviousProperties != null)
                xRunInfo.Add(this.PreviousProperties.ToXml());

            return xRunInfo;
        }

        private void ParseInternal(XElement change)
        {
            var author = change.Attribute(NS.WM + "author");

            this.Author = author.Value;

            var date = change.Attribute(NS.WM + "date");

            this.Date = DateTime.Parse(date.Value);

            var id = change.Attribute(NS.WM + "id");

            this.Id = Convert.ToDecimal(id.Value);

            var pPr = change.Element(NS.WM + "pPr");

            if (pPr != null)
            {
                this.PreviousProperties = StyleRunProperties.Parse(pPr);
            }
        }

        public static RunRevisionInformation Parse(System.Xml.Linq.XElement change)
        {
            var rev = new RunRevisionInformation();
            rev.ParseInternal(change);
            return rev;
        }
    }
}
