﻿#region MIT Licenze
//The MIT License

//Copyright (c) 2008 Sebastian Faltoni, dotnetfireball.org

//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:

//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.

//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace Fireball.OpenXML.Word
{
    /// <summary>
    /// The style section that contains defaults
    /// properties for all the document and user
    /// custom styles
    /// </summary>
    public class StyleSection : OpenXMLEntity
    {
        private StyleCollection _Styles = null;

        public StyleCollection Styles
        {
            get
            {
                return _Styles;
            }
        }

        public StyleSection()
        {
            _Styles = new StyleCollection(this);
            //_LatentStyles = new LatentStyles();
        }

        public StyleDocDefaults Defaults
        {
            get;
            set;
        }


        public LatentStyles LatentStyles
        {
            get;
            set;
        }

        public override System.Xml.Linq.XContainer ToXml()
        {
            XElement xStyles = new XElement(NS.WM + "styles",
                new XAttribute(XNamespace.Xmlns + "s", NS.WM.NamespaceName));

            if (this.Defaults != null)
                xStyles.Add(this.Defaults.ToXml());

            if (LatentStyles.Count > 0)
            {
                XElement latentStyles = new XElement(NS.WM + "latentStyles");
                /*foreach (var c in this.LatentStyles.e)
                {
                    latentStyles.Add(c.ToXml());
                }*/
                xStyles.Add(this.LatentStyles.ToXml());
            }

            foreach (var c in this.Styles)
            {
                xStyles.Add(c.ToXml());
            }

            return xStyles;
        }
    }
}
